package de.intevation.gnv.action;

import de.intevation.gnv.action.sessionmodel.SessionModel;
import de.intevation.gnv.action.sessionmodel.SessionModelFactory;

import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClient;
import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClientFactory;

import de.intevation.gnv.artifactdatabase.objects.ArtifactDescription;
import de.intevation.gnv.artifactdatabase.objects.ArtifactObject;

import de.intevation.gnv.util.ArtifactNamespaceContext;
import de.intevation.gnv.util.XMLUtils;
import de.intevation.gnv.util.XSLTransformer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import javax.xml.xpath.XPathConstants;

import org.apache.log4j.Logger;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.w3c.dom.Node;
/**
 * @author Ingo Weinzierl <ingo.weinzierl@intevation.de>
 *
 */
public class PreviousArtifactStepAction extends ArtifactDatabaseActionBase {

    public static final String XPATH_DYNAMIC_UI = "art:dynamic";
    public static final String XPATH_STATIC_UI  = "art:static";

    public static final String URL_STATE_KEY = "target";

    private static Logger logger =
        Logger.getLogger(PreviousArtifactStepAction.class);


    public PreviousArtifactStepAction() {
        super();
    }


    @Override
    public ActionForward execute(
        ActionMapping       mapping,
        ActionForm          form,
        HttpServletRequest  request,
        HttpServletResponse response
    ) throws Exception {

        String id = request.getSession().getId();

        SessionModel session =
            SessionModelFactory.getInstance().getSessionModel(request);

            // TODO check if target is reachable

            ArtifactDatabaseClientFactory f =
                ArtifactDatabaseClientFactory.getInstance();
            ArtifactDatabaseClient client   =
                f.getArtifactDatabaseClient(getLocale(request));

            ArtifactObject artifactFactory = session.getSelectedArtifactFactory();
            ArtifactObject currentArtifact = session.getCurrentArtifact();

            String target = (String) request.getParameter(URL_STATE_KEY);
            logger.debug("Step back to previous state: " + target);

            client.doAdvance(
                artifactFactory,
                currentArtifact,
                target
            );

            ArtifactDescription description = client.getCurrentStepDescription(
                artifactFactory,
                currentArtifact,
                true
            );

            Node currentUI = description.getCurrentUI();
            if (currentUI != null) {
                XSLTransformer transformer = new XSLTransformer();
                Node dynamicNode = (Node) XMLUtils.xpath(
                    currentUI,
                    XPATH_DYNAMIC_UI,
                    XPathConstants.NODE,
                    ArtifactNamespaceContext.INSTANCE
                );

                if (dynamicNode != null) {
                    String dynamicUI = transformer.transform(
                        dynamicNode, "UTF-8", request.getRealPath(
                            "WEB-INF/config/templates/describe-ui.xsl")
                    );
                    request.setAttribute("ui", dynamicUI);
                }

                Node staticNode = (Node) XMLUtils.xpath(
                    currentUI,
                    XPATH_STATIC_UI,
                    XPathConstants.NODE,
                    ArtifactNamespaceContext.INSTANCE
                );

                if (staticNode != null){
                    String staticUI = transformer.transform(
                        staticNode, "UTF-8", request.getRealPath(
                            "WEB-INF/config/templates/describe-ui-static.xsl")
                    );
                    request.setAttribute("staticui", staticUI);
                }
            }

        return super.execute(mapping, form, request, response);
    }
}
